--[[
	MrtWoo_Core: MrtWoo Core Module
	Copyright (C) 2010 Pavel Dudkovsky (mrtime@era.by)
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
]]

MrtWoo = LibStub("AceAddon-3.0"):NewAddon("MrtWoo", "AceConsole-3.0", "AceComm-3.0", "AceEvent-3.0");
local L = LibStub("AceLocale-3.0"):GetLocale("MrtWoo", true);

MrtWoo.Revision = "79";
MrtWoo.Version = "0.2.3.40000-beta2";
MrtWoo.VersionType = nil;

MrtWoo.IsNewVerExists = false;

MrtWoo.options = {
	name = "MrtWoo",
	handler = MrtWoo,
	type = 'group',
	plugins = {},	
	args = {
		verinfo = {
			type = "description",
			name = function() 
				local ver = MrtWoo.Version;
				if not string.find(MrtWoo.Version, "release") and not string.find(MrtWoo.Version, "beta") then
					ver = ver .. "(alpha)";
				end;
				return string.format(L["Current version of the addon: %s"], ver);
			end,
			order=1,			
		},
		checknew = {
			type = "toggle",
			name = L["Check new versions?"],
			desc = L["If enabled, the addon will check for new versions of the guild / group"],
			get = function() return MrtWoo.db.profile.modules.core_checknew end,
			set = function() MrtWoo.db.profile.modules.core_checknew = not MrtWoo.db.profile.modules.core_checknew end,
			order=2,
		},
	}
};

MrtWoo.defaults = {
	profile = {
		modules = {}
	}
}

function MrtWoo:OnInitialize()
	MrtWoo:Print(string.format(L["Hello from %s!"], "MrtWoo"));
	MrtWoo:RegisterChatCommand("mrtwoo", "CommandProcessor");	
	
	-- DB
	self.db = LibStub("AceDB-3.0"):New("MrtWooDB", self.defaults, "Default")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("MrtWoo", self.options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("MrtWoo", "MrtWoo")

	-- Profiles
	LibStub("AceConfig-3.0"):RegisterOptionsTable("MrtWoo-Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db))
	self.profilesFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("MrtWoo-Profiles", L["Profiles"], "MrtWoo")
	
	self:SetDefaults();
	
	self.MyName = MrtWoo:GetUnitName('player', false);
	
	if string.find(MrtWoo.Version, "release") then
		self.VersionType = 1;
	elseif string.find(MrtWoo.Version, "beta") then
		self.VersionType = 2;
	else
		self.VersionType = 3;
	end;
	
end;

function MrtWoo:SetDefaults()
	MrtWoo.defaults.profile.modules.core_checknew = true;
end;

function MrtWoo:OnEnable()
	self:RegisterComm("MrtWooVer", "CheckVersionHandler");	
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "SendVersionInfoToAll");
	self:SendVersionInfoToAll();
end;

function MrtWoo:OnDisable()
	self:UnregisterComm("MrtWooVer");
	self:UnregisterEvent("ZONE_CHANGED_NEW_AREA");
end;

MrtWoo.LastSendVerInfo = 0;

function MrtWoo:SendVersionInfoToAll()
	-- self:Print('=== MrtWoo:SendVersionInfo() ===');
	
	if not MrtWoo.db.profile.modules.core_checknew then
		return false;
	end;
	
	if GetTime() - self.LastSendVerInfo > 600 then
		local sRevision = tostring(tonumber(MrtWoo.Revision) or 0);
		
		if UnitInRaid("player") then
			self:SendCommMessage("MrtWooVer", sRevision .. "_" .. MrtWoo.Version, "RAID");
			self.LastSendVerInfo = GetTime();
		end;
		
		if IsInGuild() then
			self:SendCommMessage("MrtWooVer", sRevision .. "_" .. MrtWoo.Version, "GUILD");	
			self.LastSendVerInfo = GetTime();
		end;
		
		if UnitInParty("player") then
			self:SendCommMessage("MrtWooVer", sRevision .. "_" .. MrtWoo.Version, "PARTY");	
			self.LastSendVerInfo = GetTime();
		end;
	end;
end;

MrtWoo.SendVerInfoStatus = {};

function MrtWoo:SendVersionInfoToTarget(target)
	-- self:Print('=== MrtWoo:SendVersionInfoTo() ===');
	
	if not MrtWoo.db.profile.modules.core_checknew then
		return false;
	end;

	local isSendVerInfo = self.SendVerInfoStatus[target];
	
	if isSendVerInfo then
		return true;
	end;
			
	local iRevision = tonumber(MrtWoo.Revision) or 0;
	local sRevision = tostring(iRevision);
	self:SendCommMessage("MrtWooVer", sRevision .. "_" .. MrtWoo.Version, "WHISPER", target);
	
	self.SendVerInfoStatus[target] = true;
end;

function MrtWoo:CheckVersionHandler(prefix, message, distribution, sender)
	-- self:Print('=== MrtWoo:CheckVersionHandler() ===');

	if not MrtWoo.db.profile.modules.core_checknew then 
		return true
	end;
	
	local _,_,senderRevisionInt, senderVersionString = string.find(message, "^(%d+)_(.*)$")
	senderRevisionInt = tonumber(senderRevisionInt) or 0;
	local myRevisionInt = tonumber(MrtWoo.Revision) or 0;
			
	if (myRevisionInt > 0) and (senderRevisionInt > 0) then
		if (senderRevisionInt > myRevisionInt) then 
			local senderVersionType = nil;
			
			if string.find(senderVersionString, "release") then
				senderVersionType = 1;
			elseif string.find(senderVersionString, "beta") then
				senderVersionType = 2;
			else
				senderVersionType = 3;
			end;

			local allowUpdate = self.VersionType >= senderVersionType;
			
			if allowUpdate and not self.IsNewVerExists then
				self:Print(L["New version of addon was found in your party/guild, it's recommended to update"])
				self.IsNewVerExists = true;
			end;
			
		elseif (senderRevisionInt < myRevisionInt) then
			self:SendVersionInfoToTarget(sender);
		end;	
	end;
end;

function MrtWoo:CommandProcessor(input)
	InterfaceOptionsFrame_OpenToCategory("MrtWoo");
end;

function MrtWoo:GetUnitName(unit, showServerName)
	local name, server = UnitName(unit);
	if ( server and server ~= "" ) then
		if ( showServerName ) then
			return name.." - "..server;
		else
			return name..FOREIGN_SERVER_LABEL;
		end
	else
		return name;
	end
end